/**
 * Describe an apeman-api-fs
 * @function describe
 * @returns {Object}
 */
'use strict'

/** @lends describe */
function describe () {
  return {
    ping: {
      desc: 'Test the reachability of the api',
      params: [
        { name: 'pong', type: 'string', desc: 'Pong message to return' }
      ],
      returns: {
        type: 'string', desc: 'Pong message'
      }
    },
    glob: {
      desc: 'Resolve glob file name pattern',
      params: [
        { name: 'pattern', type: 'string', desc: 'Glob file name pattern' },
        { name: 'options', type: 'object', desc: 'Optional settings' }
      ],
      returns: {
        type: 'array', desc: 'Resolved filename patterns'
      }
    },
    exists: {
      desc: 'File exists or not',
      params: [
        { name: 'filename', type: 'string', desc: 'File name' }
      ],
      returns: {
        type: 'boolean', desc: 'Exists or not'
      }
    },
    stat: {
      desc: 'Get stat of a file',
      params: [
        { name: 'filename', type: 'string', desc: 'File name to get stat' }
      ],
      returns: {
        type: 'object', desc: 'File stat'
      }
    },
    mkdir: {
      desc: 'Generate a directory',
      params: [
        { name: 'dirname', type: 'string', desc: 'Directory name to generate' }
      ],
      returns: {
        type: 'boolean', desc: 'Generated or not'
      }
    },
    mkdirAll: {
      desc: 'Generate multiple directories',
      params: [
        { name: 'dirnames', type: 'array', desc: 'Directory names to generate' }
      ],
      returns: {
        type: 'array', desc: 'Generated directories'
      }
    },
    chmod: {
      desc: 'Change file permissions',
      params: [
        { name: 'pattern', type: 'string', desc: 'Glob filename pattern' },
        { name: 'permission', type: 'string', desc: 'Permission string (like "644")' }
      ],
      returns: {
        type: 'array', desc: 'Changed file names'
      }
    },
    chmodAll: {
      desc: 'Change file permissions',
      params: [
        { name: 'permissions', type: 'array', desc: 'Pattern and permissions' }
      ],
      returns: {
        type: 'array', desc: 'Changed file names'
      }
    },
    copy: {
      desc: 'Copy file',
      params: [
        { name: 'src', type: 'string', desc: 'Source file to copy' },
        { name: 'dest', type: 'string', desc: 'Destination file to copy' }
      ]
    }
  }
}

module.exports = describe
