/**
 * Define api of apeman-api-fs
 * @function define
 * @param {Object} [options] - Optional settings
 * @param {boolean} [options.logging] - Enable logger
 * @param {string} [options.root=process.cwd()] - Root directory path
 * @returns {Object} - API configuration
 */
'use strict';

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var co = require('co');

var _require = require('asfs');

var mkdirpAsync = _require.mkdirpAsync;
var existsAsync = _require.existsAsync;
var statAsync = _require.statAsync;

var defaults = require('defaults');
var filemode = require('filemode');
var aglob = require('aglob');
var aslogger = require('aslogger');
var filecopy = require('filecopy');
var apemanApiPath = require('apeman-api-path');
var describe = require('./describe');

var modeString = function modeString(mode) {
  return mode.toString(8);
};

/** @lends define */
function define() {
  var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  var logger = aslogger({ disabled: !options.logging });

  var _defaults = defaults(options, {
    root: process.cwd()
  });

  var root = _defaults.root;


  var path = apemanApiPath({ root: root, logging: false });

  return {
    ping: function ping() {
      var pong = arguments.length <= 0 || arguments[0] === undefined ? 'pong' : arguments[0];

      return co(regeneratorRuntime.mark(function _callee() {
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                logger.trace('pong:', pong);
                return _context.abrupt('return', pong);

              case 2:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));
    },
    glob: function glob(pattern, options) {
      var s = this;
      var args = [].concat(Array.prototype.slice.call(arguments));
      return co(regeneratorRuntime.mark(function _callee2() {
        var filenames, i, filename;
        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.next = 2;
                return aglob.apply(undefined, _toConsumableArray(args));

              case 2:
                filenames = _context2.sent;
                i = 0;

              case 4:
                if (!(i < filenames.length)) {
                  _context2.next = 12;
                  break;
                }

                filename = filenames[i];
                _context2.next = 8;
                return path.resolve(filename);

              case 8:
                filenames[i] = _context2.sent;

              case 9:
                i++;
                _context2.next = 4;
                break;

              case 12:
                return _context2.abrupt('return', filenames);

              case 13:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));
    },
    exists: function exists(filename) {
      var s = this;
      return co(regeneratorRuntime.mark(function _callee3() {
        var resolved;
        return regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _context3.next = 2;
                return path.resolve(filename);

              case 2:
                resolved = _context3.sent;
                _context3.next = 5;
                return existsAsync(resolved);

              case 5:
                return _context3.abrupt('return', _context3.sent);

              case 6:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));
    },
    stat: function stat(filename) {
      var s = this;
      return co(regeneratorRuntime.mark(function _callee4() {
        var resolved;
        return regeneratorRuntime.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                _context4.next = 2;
                return path.resolve(filename);

              case 2:
                resolved = _context4.sent;
                _context4.next = 5;
                return statAsync(resolved);

              case 5:
                return _context4.abrupt('return', _context4.sent);

              case 6:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));
    },
    mkdir: function mkdir(dirname) {
      var s = this;
      return co(regeneratorRuntime.mark(function _callee5() {
        var result;
        return regeneratorRuntime.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                _context5.next = 2;
                return s.mkdirAll([dirname]);

              case 2:
                result = _context5.sent;
                return _context5.abrupt('return', !!result[0]);

              case 4:
              case 'end':
                return _context5.stop();
            }
          }
        }, _callee5, this);
      }));
    },
    mkdirAll: function mkdirAll(dirnames) {
      var s = this;
      return co(regeneratorRuntime.mark(function _callee6() {
        var results, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, dirname, resolved, exists;

        return regeneratorRuntime.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                results = [];
                _iteratorNormalCompletion = true;
                _didIteratorError = false;
                _iteratorError = undefined;
                _context6.prev = 4;
                _iterator = dirnames[Symbol.iterator]();

              case 6:
                if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
                  _context6.next = 23;
                  break;
                }

                dirname = _step.value;
                _context6.next = 10;
                return path.resolve(dirname);

              case 10:
                resolved = _context6.sent;
                _context6.next = 13;
                return s.exists(resolved);

              case 13:
                exists = _context6.sent;

                if (!exists) {
                  _context6.next = 16;
                  break;
                }

                return _context6.abrupt('continue', 20);

              case 16:
                _context6.next = 18;
                return mkdirpAsync(resolved);

              case 18:
                logger.debug('Directory generated: ' + dirname);
                results.push(dirname);

              case 20:
                _iteratorNormalCompletion = true;
                _context6.next = 6;
                break;

              case 23:
                _context6.next = 29;
                break;

              case 25:
                _context6.prev = 25;
                _context6.t0 = _context6['catch'](4);
                _didIteratorError = true;
                _iteratorError = _context6.t0;

              case 29:
                _context6.prev = 29;
                _context6.prev = 30;

                if (!_iteratorNormalCompletion && _iterator.return) {
                  _iterator.return();
                }

              case 32:
                _context6.prev = 32;

                if (!_didIteratorError) {
                  _context6.next = 35;
                  break;
                }

                throw _iteratorError;

              case 35:
                return _context6.finish(32);

              case 36:
                return _context6.finish(29);

              case 37:
                return _context6.abrupt('return', results);

              case 38:
              case 'end':
                return _context6.stop();
            }
          }
        }, _callee6, this, [[4, 25, 29, 37], [30,, 32, 36]]);
      }));
    },
    chmod: function chmod(pattern, mode) {
      var s = this;
      return s.chmodAll(_defineProperty({}, pattern, mode));
    },
    chmodAll: function chmodAll(permissions) {
      var s = this;
      return co(regeneratorRuntime.mark(function _callee7() {
        var _this = this;

        var results, _iteratorNormalCompletion2, _didIteratorError2, _iteratorError2, _iterator2, _step2, pattern, mode, filenames, _iteratorNormalCompletion3, _didIteratorError3, _iteratorError3, _loop, _iterator3, _step3;

        return regeneratorRuntime.wrap(function _callee7$(_context8) {
          while (1) {
            switch (_context8.prev = _context8.next) {
              case 0:
                results = [];
                _iteratorNormalCompletion2 = true;
                _didIteratorError2 = false;
                _iteratorError2 = undefined;
                _context8.prev = 4;
                _iterator2 = Object.keys(permissions)[Symbol.iterator]();

              case 6:
                if (_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done) {
                  _context8.next = 40;
                  break;
                }

                pattern = _step2.value;
                mode = permissions[pattern];
                _context8.next = 11;
                return s.glob(pattern);

              case 11:
                filenames = _context8.sent;
                _iteratorNormalCompletion3 = true;
                _didIteratorError3 = false;
                _iteratorError3 = undefined;
                _context8.prev = 15;
                _loop = regeneratorRuntime.mark(function _loop() {
                  var filename, changed;
                  return regeneratorRuntime.wrap(function _loop$(_context7) {
                    while (1) {
                      switch (_context7.prev = _context7.next) {
                        case 0:
                          filename = _step3.value;
                          _context7.next = 3;
                          return filemode(filename, mode, {});

                        case 3:
                          changed = _context7.sent;

                          Object.keys(changed).forEach(function (filename) {
                            var _changed$filename = changed[filename];
                            var from = _changed$filename.from;
                            var to = _changed$filename.to;

                            logger.debug('Permission changed: ' + filename + ' ' + modeString(from) + ' to ' + modeString(to));
                            results.push(filename);
                          });

                        case 5:
                        case 'end':
                          return _context7.stop();
                      }
                    }
                  }, _loop, _this);
                });
                _iterator3 = filenames[Symbol.iterator]();

              case 18:
                if (_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done) {
                  _context8.next = 23;
                  break;
                }

                return _context8.delegateYield(_loop(), 't0', 20);

              case 20:
                _iteratorNormalCompletion3 = true;
                _context8.next = 18;
                break;

              case 23:
                _context8.next = 29;
                break;

              case 25:
                _context8.prev = 25;
                _context8.t1 = _context8['catch'](15);
                _didIteratorError3 = true;
                _iteratorError3 = _context8.t1;

              case 29:
                _context8.prev = 29;
                _context8.prev = 30;

                if (!_iteratorNormalCompletion3 && _iterator3.return) {
                  _iterator3.return();
                }

              case 32:
                _context8.prev = 32;

                if (!_didIteratorError3) {
                  _context8.next = 35;
                  break;
                }

                throw _iteratorError3;

              case 35:
                return _context8.finish(32);

              case 36:
                return _context8.finish(29);

              case 37:
                _iteratorNormalCompletion2 = true;
                _context8.next = 6;
                break;

              case 40:
                _context8.next = 46;
                break;

              case 42:
                _context8.prev = 42;
                _context8.t2 = _context8['catch'](4);
                _didIteratorError2 = true;
                _iteratorError2 = _context8.t2;

              case 46:
                _context8.prev = 46;
                _context8.prev = 47;

                if (!_iteratorNormalCompletion2 && _iterator2.return) {
                  _iterator2.return();
                }

              case 49:
                _context8.prev = 49;

                if (!_didIteratorError2) {
                  _context8.next = 52;
                  break;
                }

                throw _iteratorError2;

              case 52:
                return _context8.finish(49);

              case 53:
                return _context8.finish(46);

              case 54:
                return _context8.abrupt('return', results);

              case 55:
              case 'end':
                return _context8.stop();
            }
          }
        }, _callee7, this, [[4, 42, 46, 54], [15, 25, 29, 37], [30,, 32, 36], [47,, 49, 53]]);
      }));
    },
    copy: function copy(src, dest) {
      var options = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

      var s = this;
      return co(regeneratorRuntime.mark(function _callee8() {
        return regeneratorRuntime.wrap(function _callee8$(_context9) {
          while (1) {
            switch (_context9.prev = _context9.next) {
              case 0:
                _context9.next = 2;
                return path.resolve(src);

              case 2:
                src = _context9.sent;
                _context9.next = 5;
                return path.resolve(dest);

              case 5:
                dest = _context9.sent;
                _context9.next = 8;
                return filecopy(src, dest, {
                  mkdirp: true
                });

              case 8:
                return _context9.abrupt('return', _context9.sent);

              case 9:
              case 'end':
                return _context9.stop();
            }
          }
        }, _callee8, this);
      }));
    },

    /**
     * A hook which runs before method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @returns {Promise}
     */
    $before: function $before(methodName, params) {
      var s = this;
      var state = s.state;

      return co(regeneratorRuntime.mark(function _callee9() {
        return regeneratorRuntime.wrap(function _callee9$(_context10) {
          while (1) {
            switch (_context10.prev = _context10.next) {
              case 0:
                state.invokedAt = new Date();
                logger.info(methodName + ' started...');

              case 2:
              case 'end':
                return _context10.stop();
            }
          }
        }, _callee9, this);
      }));
    },

    /**
     * A hook which runs after method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @param {?*} returns - Return values
     * @returns {Promise}
     */
    $after: function $after(methodName, params, returns) {
      var s = this;
      var state = s.state;

      return co(regeneratorRuntime.mark(function _callee10() {
        var took;
        return regeneratorRuntime.wrap(function _callee10$(_context11) {
          while (1) {
            switch (_context11.prev = _context11.next) {
              case 0:
                took = new Date() - state.invokedAt;

                logger.info('...' + methodName + ' done! (' + took + 'ms)');

              case 2:
              case 'end':
                return _context11.stop();
            }
          }
        }, _callee10, this);
      }));
    },

    /**
     * Method specifications
     */
    $spec: describe()
  };
}

module.exports = define;
//# sourceMappingURL=data:application/json;base64,bnVsbA==