/**
 * Test case for define.
 * Runs with mocha.
 */
'use strict'

const define = require('../lib/define.js')
const assert = require('assert')
const { moduleProxy } = require('apemanmock')
const co = require('co')
const fs = require('fs')

describe('define', function () {
  let tmpDir = `${__dirname}/../tmp`
  this.timeout(3000)

  before(() => co(function * () {

  }))

  after(() => co(function * () {

  }))

  it('Ping pong', () => co(function * () {
    let module = define({
      logging: true
    })
    let api = moduleProxy(module, { ctx: { state: {} } })
    let pong = yield api.ping('The pong!')
    assert.equal(pong, 'The pong!')
  }))

  it('Check exists', () => co(function * () {
    let module = define({
      logging: true
    })
    let api = moduleProxy(module, { ctx: { state: {} } })
    assert.ok(yield api.exists(__filename))
    assert.ok(!(yield api.exists('foo/bar/__not_exists__')))
  }))

  it('Get stat', () => co(function * () {
    let module = define({
      logging: true
    })
    let api = moduleProxy(module, { ctx: { state: {} } })
    let stat = yield api.stat(__filename)
    assert.ok(stat.mode)
  }))

  it('Make directories', () => co(function * () {
    let module = define({
      logging: true
    })
    let api = moduleProxy(module, { ctx: { state: {} } })
    {
      let results = yield api.mkdirAll([
        `${tmpDir}/foo`,
        `${tmpDir}/foo/bar`
      ])
      assert.ok(results)
    }
    {
      let result = yield api.mkdir(`${tmpDir}/foo/bar/baz`)
      assert.equal(typeof result, 'boolean')
    }
    assert.ok(fs.existsSync(`${tmpDir}/foo`))
    assert.ok(fs.existsSync(`${tmpDir}/foo/bar`))
    assert.ok(fs.existsSync(`${tmpDir}/foo/bar/baz`))
  }))

  it('Change permission', () => co(function * () {
    let module = define({
      logging: true
    })
    let api = moduleProxy(module, { ctx: { state: {} } })
    {
      let results = yield api.chmod(
        `${__dirname}/../misc/mocks/mock-bin/*.js`,
        '777'
      )
      assert.ok(results)
    }
    {
      let results = yield api.chmodAll({
        [`${__dirname}/../misc/mocks/mock-bin/*.js`]: '755'
      })
      assert.ok(results)
    }
  }))

  it('Copy files', () => co(function * () {
    let module = define({
      logging: true
    })
    let api = moduleProxy(module, { ctx: { state: {} } })
    {
      let src = String(__filename)
      let dest = `${__dirname}/../tmp/testing-copy/copied01.txt`
      yield api.copy(src, dest)
      fs.existsSync(dest)
    }
  }))

  it('Compare spec and implement', () => co(function * () {
    let module = define({
      logging: false
    })
    let implNames = Object.keys(module).filter((name) => !/^\$/.test(name))
    let specNames = Object.keys(module.$spec)
    for (let implName of implNames) {
      assert.ok(!!~specNames.indexOf(implName), `${implName} is implemented, but not described in "$spec"`)
    }
    for (let specName of specNames) {
      assert.ok(!!~implNames.indexOf(specName), `${specName} is described in "$spec", but not implemented`)
    }
  }))
})

/* global describe, before, after, it */
